﻿using PoEcoDB.DB;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using static PoEcoDB.JSON.Stash;

namespace PoEcoDB
{
    internal class Program
    {
        public static string version = "1.0.0";
        static void Main(string[] args)
        {
            // Splash
            Message.Splash("PoEcoDB", "mikx");
            // Setup the base ecosystem
            if (!Directory.Exists(@"data")) { Directory.CreateDirectory(@"data"); }
            // Args Manager
            if (args.Contains("--genclass"))
            {
                Message.CMW("Getting the latest PoB Community LUA...", true, 1);
                Web.PoBLua.GetLatestLUA();
                Message.CMW("Generating the latest classes and bases...", true, 1);
                DB.ClassGen.GenClass();
                Message.CMW("Generation done.", true, 2);
                return;
            }
            if (args.Contains("--genvalue"))
            {
                Message.CMW("Getting the latest PoE.Ninja Currency Data...", true, 1);
                if (File.Exists("data/ninja/currency.json")) { File.Delete("data/ninja/currency.json"); }
                Web.PoENinja.SaveData("https://poe.ninja/api/data/" + "currencyoverview?league=" + JSON.Settings.GetLeague() + "&type=Currency&language=en", "data/ninja/currency.json");
                Web.PoENinja.GenCurrency();
                Message.CMW("Getting the latest PoE.Ninja Divination Data...", true, 1);
                if (File.Exists("data/ninja/divination.json")) { File.Delete("data/ninja/divination.json"); }
                Web.PoENinja.SaveData("https://poe.ninja/api/data/" + "itemoverview?league=" + JSON.Settings.GetLeague() + "&type=DivinationCard&language=en", "data/ninja/divination.json");
                Web.PoENinja.GenDivination();
                Message.CMW("Getting the latest PoE.Ninja Essence Data...", true, 1);
                if (File.Exists("data/ninja/essence.json")) { File.Delete("data/ninja/essence.json"); }
                Web.PoENinja.SaveData("https://poe.ninja/api/data/" + "itemoverview?league=" + JSON.Settings.GetLeague() + "&type=Essence&language=en", "data/ninja/essence.json");
                Web.PoENinja.GenEssence();
                Message.CMW("Getting the latest PoE.Ninja Scarab Data...", true, 1);
                if (File.Exists("data/ninja/scarab.json")) { File.Delete("data/ninja/scarab.json"); }
                Web.PoENinja.SaveData("https://poe.ninja/api/data/" + "itemoverview?league=" + JSON.Settings.GetLeague() + "&type=Scarab&language=en", "data/ninja/scarab.json");
                Web.PoENinja.GenFragment();
                Message.CMW("Getting the latest PoE.Ninja Fossil Data...", true, 1);
                if (File.Exists("data/ninja/fossil.json")) { File.Delete("data/ninja/fossil.json"); }
                Web.PoENinja.SaveData("https://poe.ninja/api/data/" + "itemoverview?league=" + JSON.Settings.GetLeague() + "&type=Fossil&language=en", "data/ninja/fossil.json");
                Web.PoENinja.GenFossil();
                return;
            }
            if (args.Contains("--user"))
            {
                if (args.Length > 1)
                {
                    string user = args[1];
                    Message.CMW($"User \"{user}\" processing...", true, 1);
                    Message.CMW($"Checking the database for user \"{user}\"...", true, 1);
                    if (DB.User.CheckUserExists(user))
                    {
                        var uid = DB.User.GetUserID(user);
                        Message.CMW($"User \"{user}\" exists with ID \"{uid}\".", true, 2);
                        if (args.Contains("--tabs"))
                        {
                            string token = DB.User.GetUserToken(user);
                            if (DB.User.GetUserCurrencyIndex(user, "curindex") != -1)
                            {
                                Web.Stash.DownloadStash(token, $"https://www.pathofexile.com/character-window/get-stash-items?league={JSON.Settings.GetLeague()}&tabs=1&tabIndex={DB.User.GetUserCurrencyIndex(user, "curindex")}&accountName={user}", $"data/stash/{user}.tab.{DB.User.GetUserCurrencyIndex(user, "curindex").ToString()}.json");
                                JSON.Currency.GenCurrencyTab(user, DB.User.GetUserCurrencyIndex(user, "curindex"), "curvalue", "poeco_currency");
                            }
                            if (DB.User.GetUserCurrencyIndex(user, "divindex") != -1)
                            {
                                Web.Stash.DownloadStash(token, $"https://www.pathofexile.com/character-window/get-stash-items?league={JSON.Settings.GetLeague()}&tabs=1&tabIndex={DB.User.GetUserCurrencyIndex(user, "divindex")}&accountName={user}", $"data/stash/{user}.tab.{DB.User.GetUserCurrencyIndex(user, "divindex").ToString()}.json");
                                JSON.Currency.GenCurrencyTab(user, DB.User.GetUserCurrencyIndex(user, "divindex"), "divvalue", "poeco_divination");
                            }
                            if (DB.User.GetUserCurrencyIndex(user, "essindex") != -1)
                            {
                                Web.Stash.DownloadStash(token, $"https://www.pathofexile.com/character-window/get-stash-items?league={JSON.Settings.GetLeague()}&tabs=1&tabIndex={DB.User.GetUserCurrencyIndex(user, "essindex")}&accountName={user}", $"data/stash/{user}.tab.{DB.User.GetUserCurrencyIndex(user, "essindex").ToString()}.json");
                                JSON.Currency.GenCurrencyTab(user, DB.User.GetUserCurrencyIndex(user, "essindex"), "essvalue", "poeco_essence");
                            }
                            if (DB.User.GetUserCurrencyIndex(user, "fraindex") != -1)
                            {
                                Web.Stash.DownloadStash(token, $"https://www.pathofexile.com/character-window/get-stash-items?league={JSON.Settings.GetLeague()}&tabs=1&tabIndex={DB.User.GetUserCurrencyIndex(user, "fraindex")}&accountName={user}", $"data/stash/{user}.tab.{DB.User.GetUserCurrencyIndex(user, "fraindex").ToString()}.json");
                                JSON.Currency.GenCurrencyTab(user, DB.User.GetUserCurrencyIndex(user, "fraindex"), "fravalue", "poeco_fragment");
                            }
                            if (DB.User.GetUserCurrencyIndex(user, "fosindex") != -1)
                            {
                                Web.Stash.DownloadStash(token, $"https://www.pathofexile.com/character-window/get-stash-items?league={JSON.Settings.GetLeague()}&tabs=1&tabIndex={DB.User.GetUserCurrencyIndex(user, "fosindex")}&accountName={user}", $"data/stash/{user}.tab.{DB.User.GetUserCurrencyIndex(user, "fosindex").ToString()}.json");
                                JSON.Currency.GenCurrencyTab(user, DB.User.GetUserCurrencyIndex(user, "fosindex"), "fosvalue", "poeco_fossil");
                            }
                        }
                        int tabindex = 0;
                        if (args.Contains("--tabindex"))
                        {
                            tabindex = Convert.ToInt32(args[3]);
                        } else
                        {
                            tabindex = DB.User.GetUserTabIndex(user);
                        }
                        if (args.Contains("--token"))
                        {
                            Web.Stash.DownloadStash(args[3], $"https://www.pathofexile.com/character-window/get-stash-items?league={JSON.Settings.GetLeague()}&tabs=1&tabIndex={tabindex}&accountName={user}", $"data/stash/{user}.json");
                        }      
                        int useritemcount = DB.User.GetUserItemCount(user);
                        int stashitemcount = JSON.Stash.GetUserItemCount(user);
                        JSON.Stash.UpdateUserTabRGB(user, tabindex);
                        JSON.Stash.UpdateUserTabName(user, tabindex);
                        if (stashitemcount == 0)
                        {
                            DB.User.SetUserItemCount(user, 0);
                            DB.Stash.DeleteItems(uid);
                            Message.CMW($"This tab is empty. Use a different index. (--tabindex <index>)", true, 3);
                        } else
                        {
                            if (JSON.Stash.GetUserItemCount(user) != useritemcount || DB.User.GetUserStashHash(user) != JSON.Stash.GetStashHash(user))
                            {
                                DB.User.SetUserItemCount(user, JSON.Stash.GetUserItemCount(user));
                                JSON.Stash.GenStashHash(user);
                                JSON.Stash.CleanDB(user, uid);
                                JSON.Stash.GenItemTable(user);
                            }
                            else
                            {
                                Message.CMW($"We detected no change in user stash \"{user}\".", true, 3);
                            }
                        }                                          
                    } else
                    {
                        Message.CMW($"User \"{ user}\" doesn't exists.", true, 3);
                    }
                } else
                {
                    Message.CMW($"You must specify a user with this argument.", true, 3);
                    Message.CMW($"Exemple: --user bob", true, 3);
                    Message.CMW($"Press any key to quit...", true, 1);
                    Console.ReadKey();
                    return;
                }               
                return;
            }
        }
    }
}
